@@ -22,6 +22,7 @@
 #include <libgda/gda-quark-list.h>
 #include <libgda/gda-blob-op.h>
 #include <libgda-ui/libgda-ui.h>
+#include <libgda-ui/gdaui-data-entry.h>
 
 /*
  * Fills in @bindata->data and @bindata->data_length with the contents of @value.
@@ -37,7 +38,8 @@ common_pict_load_data (PictOptions *options, const GValue *value, PictBinData *b
 	if (value) {
 		if (gda_value_is_null ((GValue *) value)) {
 			*stock = GTK_STOCK_MISSING_IMAGE;
-			g_set_error (error, 0, 0, _("No data"));
+			g_set_error (error, GDAUI_DATA_ENTRY_ERROR, GDAUI_DATA_ENTRY_INVALID_DATA_ERROR,
+				     "%s", _("No data"));
 			allok = FALSE;
 		}
 		else {
@@ -68,8 +70,8 @@ common_pict_load_data (PictOptions *options, const GValue *value, PictBinData *b
 				}
 				else {
 					*stock = GTK_STOCK_DIALOG_ERROR;
-					g_set_error (error, 0, 0,
-						     _("No data"));
+					g_set_error (error, GDAUI_DATA_ENTRY_ERROR, GDAUI_DATA_ENTRY_INVALID_DATA_ERROR,
+						     "%s", _("No data"));
 					allok = FALSE;
 				}
 			}
@@ -84,7 +86,6 @@ common_pict_load_data (PictOptions *options, const GValue *value, PictBinData *b
 						bindata->data_length = strlen ((gchar *) bindata->data);
 						break;
 					case ENCODING_BASE64: {
-#if (GLIB_MINOR_VERSION >= 12)
 						gsize out_len;
 						bindata->data = g_base64_decode (str, &out_len);
 						if (out_len > 0)
@@ -94,32 +95,29 @@ common_pict_load_data (PictOptions *options, const GValue *value, PictBinData *b
 							bindata->data = NULL;
 							bindata->data_length = 0;
 						}
-#else
-						g_warning ("Base64 enoding/decoding is not supported in the "
-							   "GLib version %d.%d.%d",
-							   glib_major_version, glib_minor_version, glib_micro_version);
-#endif
-
 						break;
 					}
 					}
 				}
 				else {
 					*stock = GTK_STOCK_MISSING_IMAGE;
-					g_set_error (error, 0, 0, _("Empty data"));
+					g_set_error (error, GDAUI_DATA_ENTRY_ERROR, GDAUI_DATA_ENTRY_INVALID_DATA_ERROR,
+						     "%s", _("Empty data"));
 					allok = FALSE;
 				}
 			}
 			else {
 				*stock = GTK_STOCK_DIALOG_ERROR;
-				g_set_error (error, 0, 0, _("Unhandled type of data"));
+				g_set_error (error, GDAUI_DATA_ENTRY_ERROR, GDAUI_DATA_ENTRY_INVALID_DATA_ERROR,
+					     "%s", _("Unhandled type of data"));
 				allok = FALSE;
 			}
 		}
 	}
 	else {
 		*stock = GTK_STOCK_MISSING_IMAGE;
-		g_set_error (error, 0, 0, _("Empty data"));
+		g_set_error (error, GDAUI_DATA_ENTRY_ERROR, GDAUI_DATA_ENTRY_INVALID_DATA_ERROR,
+			     "%s", _("Empty data"));
 		allok = FALSE;
 	}
 
@@ -208,7 +206,7 @@ common_pict_make_pixbuf (PictOptions *options, PictBinData *bindata, PictAllocat
 				bindata->data_length = 0;
 
 				*stock = GTK_STOCK_DIALOG_ERROR;
-				g_set_error (error, 0, 0,
+				g_set_error (error, GDAUI_DATA_ENTRY_ERROR, GDAUI_DATA_ENTRY_INVALID_DATA_ERROR,
 					     _("Error while deserializing data:\n%s"),
 					     loc_error && loc_error->message ? loc_error->message : _("No detail"));
 
@@ -218,7 +216,7 @@ common_pict_make_pixbuf (PictOptions *options, PictBinData *bindata, PictAllocat
 				retpixbuf = gdk_pixbuf_from_pixdata (&pixdata, FALSE, &loc_error);
 				if (!retpixbuf) {
 					*stock = GTK_STOCK_DIALOG_ERROR;
-					g_set_error (error, 0, 0,
+					g_set_error (error, GDAUI_DATA_ENTRY_ERROR, GDAUI_DATA_ENTRY_INVALID_DATA_ERROR,
 						     _("Error while interpreting data as an image:\n%s"),
 						     loc_error && loc_error->message ? loc_error->message : _("No detail"));
 					g_error_free (loc_error);
@@ -264,13 +262,11 @@ common_pict_make_pixbuf (PictOptions *options, PictBinData *bindata, PictAllocat
 				gchar *notice_msg;
 				notice_msg = g_strdup_printf (_("Error while interpreting data as an image:\n%s"),
 							      loc_error && loc_error->message ? loc_error->message : _("No detail"));
-				g_error_free (loc_error);
 				*stock = GTK_STOCK_DIALOG_WARNING;
-#if GLIB_MAJOR_VERSION == 2 && GLIB_MINOR_VERSION < 18
-				g_set_error (error, 0, 0, "%s", notice_msg);
-#else
-				g_set_error_literal (error, 0, 0, notice_msg);
-#endif
+				g_set_error_literal (error, loc_error ? loc_error->domain : GDAUI_DATA_ENTRY_ERROR,
+						     loc_error ? loc_error->code : GDAUI_DATA_ENTRY_INVALID_DATA_ERROR,
+						     notice_msg);
+				g_error_free (loc_error);
 				g_free (notice_msg);
 			}
 			
@@ -382,11 +378,9 @@ file_load_cb (GtkWidget *button, PictMenuData *menudata)
 		gdaui_set_default_path (gtk_file_chooser_get_current_folder (GTK_FILE_CHOOSER (dlg)));
 
 		if (g_file_get_contents (filename, &data, &length, &error)) {
-			if (menudata->bindata->data) {
-				g_free (menudata->bindata->data);
-				menudata->bindata->data = NULL;
-				menudata->bindata->data_length = 0;
-			}
+			g_free (menudata->bindata->data);
+			menudata->bindata->data = NULL;
+			menudata->bindata->data_length = 0;
 
 			if (menudata->options->serialize) {
 				GdkPixdata pixdata;
@@ -414,7 +408,9 @@ file_load_cb (GtkWidget *button, PictMenuData *menudata)
 
 			/* call the callback */
 			if (menudata->callback)
-				(menudata->callback) (menudata->data);
+				(menudata->callback) (menudata->bindata, menudata->data);
+			menudata->bindata->data = NULL;
+			menudata->bindata->data_length = 0;
 		}
 		else {
 			GtkWidget *msg;
@@ -453,7 +449,7 @@ add_if_writable (GdkPixbufFormat *data, PictFormat *format)
 
 		str= g_strdup_printf ("%s (%s)", gdk_pixbuf_format_get_name (data),
 				      gdk_pixbuf_format_get_description (data));
-		gtk_combo_box_append_text (format->combo, str);
+		gtk_combo_box_text_append_text (GTK_COMBO_BOX_TEXT (format->combo), str);
 		g_free (str);
 		format->formats = g_slist_append (format->formats, g_strdup (gdk_pixbuf_format_get_name (data)));
 	}
@@ -469,12 +465,12 @@ file_save_cb (GtkWidget *button, PictMenuData *menudata)
 
 	/* determine writable formats */
 	expander = gtk_expander_new (_("Image format"));
-	hbox = gtk_hbox_new (FALSE, 0);
+	hbox = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 0);
 	gtk_container_add (GTK_CONTAINER (expander), hbox);
 
 	label = gtk_label_new (_("Format image as:"));
 	gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 0);
-	combo = gtk_combo_box_new_text ();
+	combo = gtk_combo_box_text_new ();
 	gtk_box_pack_start (GTK_BOX (hbox), combo, TRUE, TRUE, 0);
 	gtk_widget_show_all (hbox);
 
@@ -484,7 +480,7 @@ file_save_cb (GtkWidget *button, PictMenuData *menudata)
 	g_slist_foreach (formats, (GFunc) add_if_writable, &pictformat);
 	g_slist_free (formats);
 
-	gtk_combo_box_prepend_text (GTK_COMBO_BOX (combo), _("Current format"));
+	gtk_combo_box_text_prepend_text (GTK_COMBO_BOX_TEXT (combo), _("Current format"));
 	gtk_combo_box_set_active (GTK_COMBO_BOX (combo), 0);
 
 	dlg = gtk_file_chooser_dialog_new (_("Select a file to save the image to"), 
@@ -607,23 +603,24 @@ void
 common_pict_init_cache (PictOptions *options)
 {
 	g_assert (!options->pixbuf_hash);
-	options->pixbuf_hash = g_hash_table_new_full (g_direct_hash, g_direct_equal, NULL, g_object_unref);
+	options->pixbuf_hash = g_hash_table_new_full (g_int_hash, g_int_equal,
+						      g_free, g_object_unref);
 }
 
 /* 
  * computes a "hash" of the binary data 
  */
-static guint
+static gint
 compute_hash (guchar *data, glong data_length)
 {
-	guint result = 0;
+	gint result = 0;
 	guchar *ptr;
 
 	if (!data)
 		return 0;
 	for (ptr = data; ptr <= data + data_length - 1; ptr++)
 		result += *ptr;
-	
+
 	return result;
 }
 
@@ -633,18 +630,17 @@ compute_hash (guchar *data, glong data_length)
 void
 common_pict_add_cached_pixbuf (PictOptions *options, const GValue *value, GdkPixbuf *pixbuf)
 {
-	guint hash;
+	gint *hash;
 	g_return_if_fail (pixbuf);
 
 	if (!options->pixbuf_hash || !value)
 		return;
-
 	else if (GDA_VALUE_HOLDS_BINARY (value)) {
 		const GdaBinary *bin;
 		bin = gda_value_get_binary (value);
-		hash = compute_hash (bin->data, bin->binary_length);
-		g_hash_table_insert (options->pixbuf_hash, GUINT_TO_POINTER (hash), pixbuf);
-		g_object_ref (pixbuf);
+		hash = g_new (guint, 1);
+		*hash = compute_hash (bin->data, bin->binary_length);
+		g_hash_table_insert (options->pixbuf_hash, hash, g_object_ref (pixbuf));
 	}
 	else if (GDA_VALUE_HOLDS_BLOB (value)) {
 		const GdaBinary *bin;
@@ -654,9 +650,9 @@ common_pict_add_cached_pixbuf (PictOptions *options, const GValue *value, GdkPix
 		if (bin) {
 			if (!bin->data && blob->op)
 				gda_blob_op_read_all (blob->op, (GdaBlob*) blob);
-			hash = compute_hash (bin->data, bin->binary_length);
-			g_hash_table_insert (options->pixbuf_hash, GUINT_TO_POINTER (hash), pixbuf);
-			g_object_ref (pixbuf);
+			hash = g_new (guint, 1);
+			*hash = compute_hash (bin->data, bin->binary_length);
+			g_hash_table_insert (options->pixbuf_hash, hash, g_object_ref (pixbuf));
 		}
 	}
 }
@@ -668,7 +664,7 @@ GdkPixbuf *
 common_pict_fetch_cached_pixbuf (PictOptions *options, const GValue *value)
 {
 	GdkPixbuf *pixbuf = NULL;
-	guint hash;
+	gint hash;
 
 	if (!options->pixbuf_hash)
 		return NULL;
@@ -679,7 +675,7 @@ common_pict_fetch_cached_pixbuf (PictOptions *options, const GValue *value)
 		bin = gda_value_get_binary (value);
 		if (bin) {
 			hash = compute_hash (bin->data, bin->binary_length);
-			pixbuf = g_hash_table_lookup (options->pixbuf_hash, GUINT_TO_POINTER (hash));
+			pixbuf = g_hash_table_lookup (options->pixbuf_hash, &hash);
 		}
 	}
 	else if (GDA_VALUE_HOLDS_BLOB (value)) {
@@ -691,7 +687,7 @@ common_pict_fetch_cached_pixbuf (PictOptions *options, const GValue *value)
 			if (!bin->data && blob->op)
 				gda_blob_op_read_all (blob->op, (GdaBlob*) blob);
 			hash = compute_hash (bin->data, bin->binary_length);
-			pixbuf = g_hash_table_lookup (options->pixbuf_hash, GUINT_TO_POINTER (hash));
+			pixbuf = g_hash_table_lookup (options->pixbuf_hash, &hash);
 		}
 	}
 
@@ -756,12 +752,7 @@ common_pict_get_value (PictBinData *bindata, PictOptions *options, GType gtype)
 						 bindata->data_length);
 				break;
 			case ENCODING_BASE64: 
-#if (GLIB_MINOR_VERSION >= 12)
 				str = g_base64_encode (bindata->data, bindata->data_length);
-#else
-				g_warning ("Base64 enoding/decoding is not supported in the GLib version %d.%d.%d",
-					   glib_major_version, glib_minor_version, glib_micro_version);
-#endif
 				break;
 			}
 			
