@@ -1,3 +1,20 @@
+/*
+ * Copyright (C) 2011 Vivien Malerba <malerba@gnome-db.org>
+ *
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License
+ * as published by the Free Software Foundation; either version 2
+ * of the License, or (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
+ */
 #include <gtk/gtk.h>
 #include <string.h>
 #include <libgda-ui/libgda-ui.h>
@@ -197,9 +214,9 @@ create_window (GdaConnection *cnc)
 
 	/* container */
 	GtkWidget *vb, *hp;
-	vb = gtk_vbox_new (FALSE, 0);
+	vb = gtk_box_new (GTK_ORIENTATION_VERTICAL, 0);
 	gtk_container_add (GTK_CONTAINER (win), vb);
-	hp = gtk_hpaned_new ();
+	hp = gtk_paned_new (GTK_ORIENTATION_HORIZONTAL);
         gtk_box_pack_start (GTK_BOX (vb), hp, TRUE, TRUE, 0);
 
 	GdaTree *tree;
