@@ -175,6 +175,7 @@ static void
 open_button_clicked_cb (GtkWidget *button, GdauiEntryImport *mgtxt)
 {
 	gint res;
+
 	if (! mgtxt->priv->import_dlg) {
 		GtkWidget *dialog, *wid;
 		dialog = gtk_dialog_new_with_buttons (_("Data set import from file"),
@@ -187,11 +188,7 @@ open_button_clicked_cb (GtkWidget *button, GdauiEntryImport *mgtxt)
 						      NULL);
 		gtk_window_set_default_size (GTK_WINDOW (dialog), 620, 450);
 		wid = gdaui_data_import_new ();
-#if GTK_CHECK_VERSION(2,18,0)
-		gtk_container_add (GTK_CONTAINER (gtk_dialog_get_content_area (GTK_DIALOG (dialog))), wid);
-#else
-		gtk_container_add (GTK_CONTAINER (GTK_DIALOG (dialog)->vbox), wid);
-#endif
+		gtk_box_pack_start (GTK_BOX (gtk_dialog_get_content_area (GTK_DIALOG (dialog))), wid, TRUE, TRUE, 0);
 		gtk_widget_show_all (dialog);
 		mgtxt->priv->import_dlg = dialog;
 		mgtxt->priv->import_wid = wid;
@@ -245,7 +242,7 @@ create_entry (GdauiEntryWrapper *mgwrap)
 	mgtxt = GDAUI_ENTRY_IMPORT (mgwrap);
 	g_return_val_if_fail (mgtxt->priv, NULL);
 
-	hbox = gtk_hbox_new (FALSE, 0);
+	hbox = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 0);
 
 	mgtxt->priv->label = GTK_LABEL (gtk_label_new (_("No data set")));
 	gtk_misc_get_alignment (GTK_MISC (mgtxt->priv->label), NULL, &ya);
