@@ -1,24 +1,20 @@
-/* GNOME DB library
- * Copyright (C) 1999 - 2011 The GNOME Foundation.
+/*
+ * Copyright (C) 2009 - 2011 Vivien Malerba <malerba@gnome-db.org>
+ * Copyright (C) 2010 David King <davidk@openismus.com>
  *
- * AUTHORS:
- *      Rodrigo Moya <rodrigo@gnome-db.org>
- *      Vivien Malerba <malerba@gnome-db.org>
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License
+ * as published by the Free Software Foundation; either version 2
+ * of the License, or (at your option) any later version.
  *
- * This Program is free software; you can redistribute it and/or
- * modify it under the terms of the GNU General Public License as
- * published by the Free Software Foundation; either version 2 of the
- * License, or (at your option) any later version.
- *
- * This Library is distributed in the hope that it will be useful,
+ * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Library General Public License for more details.
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
  *
- * You should have received a copy of the GNU Library General Public
- * License along with this Library; see the file COPYING.LIB.  If not,
- * write to the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
  */
 
 #include <glib/gi18n-lib.h>
@@ -41,6 +37,7 @@
 #include "../browser-window.h"
 #include <libgda-ui/internal/popup-container.h>
 #include "../support.h"
+#include "../common/widget-overlay.h"
 
 #define QUERY_EDITOR_LANGUAGE_SQL "gda-sql"
 #define COLOR_ALTER_FACTOR 1.8
@@ -95,6 +92,10 @@ struct _QueryEditorPrivate {
 	GtkTreeView *completion_treeview;
 	GtkCellRenderer *completion_renderer;
 	GtkWidget *completion_sw;
+
+	/* tooltip */
+	GtkWidget *ovl;
+	GtkWidget *tooltip_widget;
 };
 
 static void query_editor_class_init (QueryEditorClass *klass);
@@ -538,7 +539,7 @@ event (GtkWidget *text_view, GdkEvent *ev, QueryEditor *editor)
 					g_free (editor->priv->current_state_text);
 					editor->priv->current_state_text = query_editor_get_all_text (editor);
 				}
-				else if (editor->priv->current_state >= editor->priv->states->len)
+				else if (editor->priv->current_state >= (gint)editor->priv->states->len)
 					i = editor->priv->states->len - 1; /* last stored state */
 				else if (editor->priv->current_state > 0)
 					i = editor->priv->current_state - 1;
@@ -563,7 +564,7 @@ event (GtkWidget *text_view, GdkEvent *ev, QueryEditor *editor)
 			 (evkey->keyval == GDK_KEY_Down) &&
 			 editor->priv->states) {
 			if (editor->priv->states->len > 0) {
-				if (editor->priv->current_state < editor->priv->states->len - 1) {
+				if (editor->priv->current_state < (gint)editor->priv->states->len - 1) {
 					gchar *tmp;
 					editor->priv->current_state ++;
 					tmp = g_array_index (editor->priv->states, gchar*, editor->priv->current_state);
@@ -662,9 +663,12 @@ query_editor_init (QueryEditor *editor, G_GNUC_UNUSED QueryEditorClass *klass)
 	int tab = 8;
 	gboolean highlight = TRUE;
 	gboolean showlinesno = FALSE;
+	GtkWidget *ovl, *wid;
 
 	g_return_if_fail (QUERY_IS_EDITOR (editor));
 
+	gtk_orientable_set_orientation (GTK_ORIENTABLE (editor), GTK_ORIENTATION_VERTICAL);
+
 	/* allocate private structure */
 	editor->priv = g_new0 (QueryEditorPrivate, 1);
 	editor->priv->mode = QUERY_EDITOR_READWRITE;
@@ -679,12 +683,31 @@ query_editor_init (QueryEditor *editor, G_GNUC_UNUSED QueryEditorClass *klass)
 	editor->priv->completion_popup = NULL;
 
 	/* set up widgets */
+	ovl = widget_overlay_new ();
+	editor->priv->ovl = ovl;
+	gtk_box_pack_start (GTK_BOX (editor), ovl, TRUE, TRUE, 2);
+
 	editor->priv->scrolled_window = gtk_scrolled_window_new (NULL, NULL);
         gtk_scrolled_window_set_shadow_type (GTK_SCROLLED_WINDOW (editor->priv->scrolled_window),
 					     GTK_SHADOW_ETCHED_OUT);
 	gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (editor->priv->scrolled_window),
 					GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
-	gtk_box_pack_start (GTK_BOX (editor), editor->priv->scrolled_window, TRUE, TRUE, 2);
+	gtk_container_add (GTK_CONTAINER (ovl), editor->priv->scrolled_window);
+	widget_overlay_set_child_props (WIDGET_OVERLAY (ovl), editor->priv->scrolled_window,
+					WIDGET_OVERLAY_CHILD_HALIGN, WIDGET_OVERLAY_ALIGN_FILL,
+					WIDGET_OVERLAY_CHILD_VALIGN, WIDGET_OVERLAY_ALIGN_FILL,
+					-1);
+
+	wid = gtk_label_new ("");
+	editor->priv->tooltip_widget = wid;
+	gtk_label_set_markup (GTK_LABEL (wid), QUERY_EDITOR_TOOLTIP);
+	gtk_container_add (GTK_CONTAINER (ovl), wid);
+	widget_overlay_set_child_props (WIDGET_OVERLAY (ovl), wid,
+					WIDGET_OVERLAY_CHILD_HALIGN, WIDGET_OVERLAY_ALIGN_CENTER,
+					WIDGET_OVERLAY_CHILD_VALIGN, WIDGET_OVERLAY_ALIGN_CENTER,
+					WIDGET_OVERLAY_CHILD_ALPHA, .8,
+					WIDGET_OVERLAY_CHILD_TOOLTIP, TRUE,
+					-1);
 
 #ifdef HAVE_GTKSOURCEVIEW
 	editor->priv->text = gtk_source_view_new ();
@@ -697,7 +720,6 @@ query_editor_init (QueryEditor *editor, G_GNUC_UNUSED QueryEditorClass *klass)
 #endif
 
 	gtk_container_add (GTK_CONTAINER (editor->priv->scrolled_window), editor->priv->text);
-	gtk_widget_set_tooltip_markup (editor->priv->text, QUERY_EDITOR_TOOLTIP);
 	g_signal_connect (editor->priv->text, "event", 
 			  G_CALLBACK (event), editor);
 	g_signal_connect (gtk_text_view_get_buffer (GTK_TEXT_VIEW (editor->priv->text)), "changed", 
@@ -734,7 +756,7 @@ query_editor_init (QueryEditor *editor, G_GNUC_UNUSED QueryEditorClass *klass)
 
 	create_tags_for_sql (editor, QUERY_EDITOR_LANGUAGE_SQL);
 
-	gtk_widget_show_all (editor->priv->scrolled_window);
+	gtk_widget_show_all (ovl);
 
 	/* timeout function to update timestamps */
 	editor->priv->ts_timeout_id = 0;
@@ -745,14 +767,13 @@ query_editor_map (GtkWidget *widget)
 {
 	GTK_WIDGET_CLASS (parent_class)->map (widget);
 	if (QUERY_EDITOR (widget)->priv->mode == QUERY_EDITOR_HISTORY) {
-		GtkStyle *style;
-		GdkColor color;
-		style = gtk_widget_get_style (widget);
-		color = style->bg[GTK_STATE_NORMAL];
-		color.red += (65535 - color.red) / COLOR_ALTER_FACTOR;
-		color.green += (65535 - color.green) / COLOR_ALTER_FACTOR;
-		color.blue += (65535 - color.blue) / COLOR_ALTER_FACTOR;
-		gtk_widget_modify_base (QUERY_EDITOR (widget)->priv->text, GTK_STATE_NORMAL, &color);
+		GtkStyleContext* style_context = gtk_widget_get_style_context (widget);
+		GdkRGBA color;
+		gtk_style_context_get_background_color (style_context, GTK_STATE_FLAG_NORMAL, &color);
+		color.red += (1.0 - color.red) / COLOR_ALTER_FACTOR;
+		color.green += (1.0 - color.green) / COLOR_ALTER_FACTOR;
+		color.blue += (1.0 - color.blue) / COLOR_ALTER_FACTOR;
+		gtk_widget_override_background_color (QUERY_EDITOR (widget)->priv->text, GTK_STATE_FLAG_NORMAL, &color);
 	}
 }
 
@@ -846,7 +867,7 @@ query_editor_get_type (void)
 			(GInstanceInitFunc) query_editor_init,
 			0
 		};
-		type = g_type_register_static (GTK_TYPE_VBOX, "QueryEditor", &info, 0);
+		type = g_type_register_static (GTK_TYPE_BOX, "QueryEditor", &info, 0);
 	}
 	return type;
 }
@@ -931,27 +952,28 @@ query_editor_set_mode (QueryEditor *editor, QueryEditorMode mode)
 		gtk_widget_set_tooltip_markup (editor->priv->text, NULL);
 		gtk_text_view_set_editable (GTK_TEXT_VIEW (editor->priv->text), FALSE);
 		gtk_text_view_set_cursor_visible (GTK_TEXT_VIEW (editor->priv->text), FALSE);
+		gtk_widget_destroy (editor->priv->tooltip_widget);
+		editor->priv->tooltip_widget = NULL;
 		break;
 	default:
 		g_assert_not_reached ();
 	}
 
 	if (mode == QUERY_EDITOR_HISTORY) {
-		GtkStyle *style;
-		GdkColor color;
-		style = gtk_widget_get_style ((GtkWidget*) editor);
-		color = style->bg[GTK_STATE_NORMAL];
-		color.red += (65535 - color.red) / COLOR_ALTER_FACTOR;
-		color.green += (65535 - color.green) / COLOR_ALTER_FACTOR;
-		color.blue += (65535 - color.blue) / COLOR_ALTER_FACTOR;
-		gtk_widget_modify_base (editor->priv->text, GTK_STATE_NORMAL, &color);
+		GtkStyleContext *style_context = gtk_widget_get_style_context (GTK_WIDGET (editor));
+		GdkRGBA color;
+		gtk_style_context_get_background_color (style_context, GTK_STATE_FLAG_NORMAL, &color);
+		color.red += (1.0 - color.red) / COLOR_ALTER_FACTOR;
+		color.green += (1.0 - color.green) / COLOR_ALTER_FACTOR;
+		color.blue += (1.0 - color.blue) / COLOR_ALTER_FACTOR;
+		gtk_widget_override_background_color (editor->priv->text, GTK_STATE_FLAG_NORMAL, &color);
 
 		editor->priv->hash = g_hash_table_new_full (NULL, NULL, NULL,
 							    (GDestroyNotify) hist_item_data_unref);
 	}
 	else {
-		gtk_widget_modify_base (editor->priv->text,
-					GTK_STATE_NORMAL, NULL);
+		gtk_widget_override_background_color (editor->priv->text,
+					GTK_STATE_FLAG_NORMAL, NULL);
 	}
 }
 
@@ -1138,6 +1160,8 @@ focus_on_hist_data (QueryEditor *editor, HistItemData *hdata)
 		GtkTextIter iter;
 
 		buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW (editor->priv->text));
+		if (editor->priv->hist_focus)
+			hist_item_data_unref (editor->priv->hist_focus);
 		editor->priv->hist_focus = hist_item_data_ref (hdata);
 		if (hdata->item)
 			g_object_set (G_OBJECT (hdata->tag),
@@ -1202,7 +1226,7 @@ get_date_format (time_t time)
 /**
  * query_editor_start_history_batch
  * @editor: a #QueryEditor widget.
- * @hist_batch: a #QueryEditorHistoryBatch to add, or %NULL
+ * @hist_batch: (allow-none): a #QueryEditorHistoryBatch to add, or %NULL
  *
  * @hist_hash ref is _NOT_ stolen here
  */
@@ -1224,6 +1248,10 @@ query_editor_start_history_batch (QueryEditor *editor, QueryEditorHistoryBatch *
 		
 		hist_batch = query_editor_history_batch_new (run_time, NULL);
 	}
+	else {
+		if (g_slist_find (editor->priv->batches_list, hist_batch))
+			return;
+	}
 
 	/* update editor->priv->insert_into_batch */
 	if (editor->priv->insert_into_batch)
@@ -1315,7 +1343,7 @@ timestamps_update_cb (QueryEditor *editor)
 /**
  * query_editor_add_history_item
  * @editor: a #QueryEditor widget.
- * @hist_item: a #QueryEditorHistoryItem to add, or %NULL
+ * @hist_item: (allow-none): a #QueryEditorHistoryItem to add, or %NULL
  *
  * Adds some text. @text_data is useful only if @editor's mode is HISTORY, it will be ignored
  * otherwise.
@@ -1390,6 +1418,7 @@ query_editor_add_history_item (QueryEditor *editor, QueryEditorHistoryItem *hist
 /**
  * query_editor_get_current_history_item
  * @editor: a #QueryEditor widget.
+ * @out_in_batch: (allow-none): a pointer to store the #QueryEditorHistoryBatch the returned item is in, or %NULL
  *
  * Get the current selected #QueryEditorHistoryItem
  * passed to query_editor_add_history_item().
@@ -1397,13 +1426,19 @@ query_editor_add_history_item (QueryEditor *editor, QueryEditorHistoryItem *hist
  * Returns: a #QueryEditorHistoryItem pointer, or %NULL
  */
 QueryEditorHistoryItem *
-query_editor_get_current_history_item (QueryEditor *editor)
+query_editor_get_current_history_item (QueryEditor *editor, QueryEditorHistoryBatch **out_in_batch)
 {
 	g_return_val_if_fail (QUERY_IS_EDITOR (editor), NULL);
 	g_return_val_if_fail (editor->priv->mode == QUERY_EDITOR_HISTORY, NULL);
 
-	if (editor->priv->hist_focus)
+	if (out_in_batch)
+		*out_in_batch = NULL;
+
+	if (editor->priv->hist_focus) {
+		if (out_in_batch)
+			*out_in_batch = editor->priv->hist_focus->batch;
 		return editor->priv->hist_focus->item;
+	}
 	else
 		return NULL;
 }
@@ -1416,7 +1451,7 @@ query_editor_get_current_history_item (QueryEditor *editor)
  * a #QueryEditorHistoryItem, but on the #QueryEditorHistoryBatch which was last
  * set by a call to query_editor_start_history_batch().
  * 
- * Returns: a #QueryEditorHistoryItem pointer, or %NULL
+ * Returns: a #QueryEditorHistoryBatch pointer, or %NULL
  */
 QueryEditorHistoryBatch *
 query_editor_get_current_history_batch (QueryEditor *editor)
@@ -1576,7 +1611,7 @@ query_editor_del_history_batch (QueryEditor *editor, QueryEditorHistoryBatch *ba
 	focus_on_hist_data (editor, focus);
 
 	/* remove all history items */
-	for (list =  batch->hist_items; list; list =  batch->hist_items) {
+	for (list =  batch->hist_items; list; list = batch->hist_items) {
 		hdata = g_hash_table_lookup (editor->priv->hash, list->data);
 		g_assert (hdata);
 
@@ -1828,6 +1863,9 @@ query_editor_history_batch_ref (QueryEditorHistoryBatch *qib)
 {
 	g_return_val_if_fail (qib, NULL);
 	qib->ref_count++;
+#ifdef QUERY_EDITOR_DEBUG
+	g_print ("%s (%p) ref count:%d\n", __FUNCTION__, qib, qib->ref_count);
+#endif
 	return qib;
 }
 
@@ -1836,6 +1874,9 @@ query_editor_history_batch_unref (QueryEditorHistoryBatch *qib)
 {
 	g_return_if_fail (qib);
 	qib->ref_count--;
+#ifdef QUERY_EDITOR_DEBUG
+	g_print ("%s (%p) ref count:%d\n", __FUNCTION__, qib, qib->ref_count);
+#endif
 	if (qib->ref_count <= 0) {
 		if (qib->hist_items) {
 			g_slist_foreach (qib->hist_items, (GFunc) query_editor_history_item_unref, NULL);
@@ -1903,6 +1944,9 @@ query_editor_history_item_ref (QueryEditorHistoryItem *qih)
 {
 	g_return_val_if_fail (qih, NULL);
 	qih->ref_count++;
+#ifdef QUERY_EDITOR_DEBUG
+	g_print ("%s (%p) ref count:%d\n", __FUNCTION__, qih, qih->ref_count);
+#endif
 	return qih;
 }
 
@@ -1911,6 +1955,9 @@ query_editor_history_item_unref (QueryEditorHistoryItem *qih)
 {
 	g_return_if_fail (qih);
 	qih->ref_count--;
+#ifdef QUERY_EDITOR_DEBUG
+	g_print ("%s (%p) ref count:%d\n", __FUNCTION__, qih, qih->ref_count);
+#endif
 	if (qih->ref_count <= 0) {
 		g_free (qih->sql);
 		if (qih->result)
@@ -1945,6 +1992,9 @@ hist_item_data_ref (HistItemData *hdata)
 {
 	g_return_val_if_fail (hdata, NULL);
 	hdata->ref_count ++;
+#ifdef QUERY_EDITOR_DEBUG
+	g_print ("%s (%p) ref count:%d\n", __FUNCTION__, hdata, hdata->ref_count);
+#endif
 	return hdata;
 }
 
@@ -1953,6 +2003,9 @@ hist_item_data_unref (HistItemData *hdata)
 {
 	g_return_if_fail (hdata);
 	hdata->ref_count --;
+#ifdef QUERY_EDITOR_DEBUG
+	g_print ("%s (%p) ref count:%d\n", __FUNCTION__, hdata, hdata->ref_count);
+#endif
 	if (hdata->ref_count <= 0) {
 		if (hdata->batch)
 			query_editor_history_batch_unref (hdata->batch);
