@@ -83,9 +83,6 @@ typedef enum {
 	BROWSER_ICON_QUERY,
 	BROWSER_ICON_ACTION,
 	
-	BROWSER_ICON_GRID,
-	BROWSER_ICON_FORM,
-
 	BROWSER_ICON_MENU_INDICATOR,
 
 	BROWSER_ICON_LDAP_ENTRY,
